package id

import (
	"fmt"
)

const (
	Role        = "role"
	RolesPrefix = "roles"
)

var _ Descriptor = &RoleId{}

type RoleId struct {
	SpaceId
	RoleID string `json:"role_id,omitempty" bson:"role_id,omitempty"`
}

func (id *RoleId) New() Descriptor {
	return &RoleId{}
}

func (id *RoleId) Type() string { return Role }

func (id *RoleId) String() string {
	return Join(id.SpaceId.String(), RolesPrefix, id.RoleID)
}

func (id *RoleId) FromParts(parts []string) error {
	if len(parts) != 4 || parts[2] != RolesPrefix {
		return ErrInvalidID
	}
	if err := id.SpaceId.FromParts(parts[:2]); err != nil {
		return err
	}
	id.RoleID = parts[3]
	return nil
}

func (id *RoleId) Map() map[string]any {
	m := id.SpaceId.Map()
	m["role_id"] = id.RoleID
	m["type"] = Role
	return m
}

func (id *RoleId) FromMap(m map[string]any) error {
	id.RoleID = m["role_id"].(string)
	if id.RoleID == "" {
		return fmt.Errorf("%w: RoleID required", ErrInvalidID)
	}
	return id.SpaceId.FromMap(m)
}

func (id *RoleId) Validate() error {
	if id.RoleID == "" {
		return fmt.Errorf("%w: RoleID required", ErrInvalidID)
	}

	return id.SpaceId.Validate()
}

func NewRoleId(spaceID, id string) *ObjectId {
	return &ObjectId{Descriptor: &RoleId{SpaceId: SpaceId{SpaceID: spaceID}, RoleID: id}}
}
